# -*- coding: utf-8 -*-
# Description:
# A log parser for the Acunetix XML Format
#
# Authors:
# Benoît Allard <benoit.allard@greenbone.net>
#
# Copyright:
# Copyright (C) 2015 Greenbone Networks GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

""" A log parser for the Acunetix XML Format """

import xml.etree.ElementTree as ET

SEVERITIES = {
    'info': 0.0,
    'low': 3.0,
    'medium': 5.0,
    'high': 8.0,
}


class Parser(object):

    def __init__(self, filename):
        self.tree = ET.parse(filename)
        self.root = self.tree.getroot()

    def getScans(self):
        for item in self.root.findall("Scan"):
            yield Scan(item)


class Scan(object):
    def __init__(self, xml):
        self.xml = xml
        self.items = [ReportItem(i) for i in
                      self.xml.findall("ReportItems/ReportItem")]

    def getResults(self, AcunetixSeverity=True):
        for item in self.items:
            yield item.asResult(AcunetixSeverity)


class ReportItem(object):
    def __init__(self, xml):
#        self.id = int(xml.get("id", -1))
        self.name = xml.findtext("Name")
        self.details = xml.findtext("Details")
        self.affects = xml.findtext("Affects")
        self.severity = xml.findtext("Severity")
        self.type = xml.findtext("Type")
        self.impact = xml.findtext("Impact")
        self.description = xml.findtext("Description")
        self.recommendation = xml.findtext("Recommendation")
        self.CWE = None
        cwe = xml.find("CWE")
        if cwe is not None:
            self.CWE = int(cwe.get("id"))
        # Not sure which one of the two syntax is used ...
#        self.CVEs = [x.findtext('Id') for x in xml.findall("CVEList/CVE")]
        self.CVEs = [x.text for x in xml.findall("CVEList/CVE/Id")]
        self.CVSS = xml.findtext("CVSS/Descriptor")
        self.CVSSScore = float(xml.findtext("CVSS/Score", 0))
        self.references = [(x.findtext('Database'), x.findtext('URL')) for x in xml.findall('References/Reference')]

    def __str__(self):
        return "%s on %s [Severity: %.1f (%s)]" % (self.name, self.affects, self.CVSSScore, self.severity)

    def asResult(self, AcunetixSeverity):
        """ Returns a dict that can be unrolled into the add_scan_alarm method """
        value = ["This vulnerability affects the following URL: %s" % self.affects, '']
        value.extend(["Type:", self.type, ''])
        value.extend(["Description:", self.description, ''])
        value.extend(["Recommendation:", self.recommendation])
        if self.details:
            value.extend(['', "Details:", self.details])
        if self.CVEs:
            value.extend(['', "CVE:"] + self.cves)
        if self.references:
            value.extend(['', "References:"])
            for ref in self.references:
                value.append("%s: %s" % ref)
        if AcunetixSeverity:
            value.extend(['', "The CVSS Severity for this Vulnerability is: %.1f (%s)." % (self.CVSSScore, self.CVSS)])
            severity = SEVERITIES[self.severity]
        else:
            value.extend(['', "Acunetix is gives to this Vulnerability a severity of '%s'." % (self.severity)])
            severity = self.CVSSScore
        return dict(
            name=self.name,
            value='\n'.join(value),
            severity=severity,
        )
