# -*- coding: utf-8 -*-
# Description:
# Core the OSP Acunetix Server
#
# Authors:
# Benoît Allard <benoit.allard@greenbone.net>
#
# Copyright:
# Copyright (C) 2015 Greenbone Networks GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import logging

from ospd.ospd import OSPDaemon
from ospd.misc import main as daemon_main
from ospd_acunetix import __version__

from ospd_acunetix.acunetix import Acunetix

logger = logging.getLogger(__name__)

WRAPPER_DESC = \
"""This scanner starts Acunetix WVS to scan the target systems
and returns the results.
"""

WRAPPER_SETTINGS = {
    'path': {
         'type': 'string',
         'name': 'Install path',
         'default': r"c:\Program Files\Acunetix\Web Vulnerability Scanner 10",
         'description': "Installation path of the Acunetix Scanner. The wvs_console.exe application will be searched there.",
    },
    'profile': {
        'type': 'string',
        'name': 'Scan Profile',
        'default': '',
        'description': "Scan Profile to be used, they must be defined in the GUI Application",
    },
    'severity': {
         'type': 'selection',
         'name': 'Severity source',
         'default': 'acunetix|cvss',
         'description': 'Whether to use the severity given by Acunetix, or by the CVSS Vector. '
                        'The not selected value will be provided in the results text as information.',
    },
}


class OSPDWrapper(OSPDaemon):

    def __init__(self, certfile, keyfile, cafile):
        super(OSPDWrapper, self).__init__(certfile=certfile, keyfile=keyfile, cafile=cafile)
        self.server_version = __version__
        self.scanner_info['name'] = 'Acunetix'
        self.scanner_info['version'] = 'depends on the version of the installed scanner'
        self.scanner_info['description'] = WRAPPER_DESC
        for name, params in WRAPPER_SETTINGS.items():
            self.add_scanner_param(name, params)

    def check(self):
        # If we hard-code the path to the scanner, we could check it there
        return True

    def exec_scan(self, scan_id, target):
        options = self.get_scan_options(scan_id)

        client = Acunetix(options['path'])
        log = client.run_scan(target, options['profile'])

        scans = list(log.getScans())
        if len(scans) != 1:
            self.add_scan_error(scan_id, host=target, value="Unexpected amount of scans in the log: %d" % len(scans))
            return
        scan = scans[0]
        for result in scan.getResults(options['severity'] == 'acunetix'):
            self.add_scan_alarm(scan_id, host=target, **result)

        if logger.getEffectiveLevel() <= logging.DEBUG:
            logger.debug("Running in debug-mode, not cleaning up the temporary directory.")
        else:
            client.cleanup()


def main():
    daemon_main('OSPD - Acunetix wrapper', OSPDWrapper)
